/*
 *
 *	Atari specific definitions
 *
 */
 
#include <osbind.h>
#include <falcon.h>
#include <sysvars.h>

#include <gemdefs.h>
#include <vdibind.h>
#include <aesbind.h>

/* Some system adresses */

#define VIDEL_COLOR0	(*(volatile long *)0xFFFF9800)
#define _P_COOKIES	(*(volatile long *)0x000005a0)

/* --- Cookies --- */

#define COOKIE_MCH (('_'<<24)|('M'<<16)|('C'<<8)|'H')
#define COOKIE_VDO (('_'<<24)|('V'<<16)|('D'<<8)|'O')
#define COOKIE_SND (('_'<<24)|('S'<<16)|('N'<<8)|'D')

/* Values for _MCH cookie */

#define MCH_TT	2
#define MCH_F30	3

/* Values for _VDO cookie */

#define VDO_TT	2
#define VDO_F30	3

/* Bits masks for _SND cookie */

#define SND_YM		0 	/* Yamaha ym2149 */
#define SND_DMA8	1 	/* STE/TT dma sound */
#define SND_DMAREC	2	/* Dma recording functions with xbios */
#define SND_CODEC16	3	/* Falcon 16 bits codec */
#define SND_DSP56K	4 	/* Falcon DSP56001 */

/* variables */

extern int cookie_mch;
extern int cookie_vdo;
extern int cookie_snd;
extern int joypads;	/* 0 : off , 1 : on */

void I_ScanCookies(void);

/* --- Sound definitions --- */

extern int atari_sound;	/* use DMA sound ? */

extern char *atari_dmabuffer;

/* Output coef for freq */
/* = (11025*65536)/machine freq */

extern int atari_sndcoef;

/* Length of DMA buffer */

extern int atari_sndlength;

#define DMAFREQ_TT	12585
#define DMAFREQ_F30	12292

#define TIMERA_FREQ	35

/* --- Video definitions --- */

/* TT 320*480*8bits mode */

#define TT_LOW	0x700

/* --- Chunky to planar conversion --- */ 

void init_c2p(void);
void convert_c2p(unsigned char *src,unsigned char *dest,long width,long height,long dblligne);

/* --- Control definitions --- */

void I_InitInterrupts(void);
void I_ShutdownInterrupts(void);

extern unsigned char atari_clavier[128];
extern short atari_CtrlCPressed;

extern short atari_mousex;
extern short atari_mousey;
extern short atari_mouseb;

extern short atari_joystick;

#define JS_UP 		(1<<0)
#define JS_DOWN		(1<<1)
#define JS_LEFT		(1<<2)
#define JS_RIGHT	(1<<3)
#define JS_FIRE		(1<<7)

extern long atari_joypad0;
extern long atari_joypad1;

#define JP_PAUSE	(1<<0)
#define JP_FIRE0	(1<<1)
#define JP_FIRE1	(1<<3)
#define JP_FIRE2	(1<<5)
#define JP_OPTION	(1<<7)
#define JP_UP		(1<<16)
#define JP_DOWN		(1<<17)
#define JP_LEFT		(1<<18)
#define JP_RIGHT	(1<<19)
#define JP_KASTER	(1<<20)
#define JP_K7		(1<<21)
#define JP_K4		(1<<22)
#define JP_K1		(1<<23)
#define JP_K0		(1<<24)
#define JP_K8		(1<<25)
#define JP_K5		(1<<26)
#define JP_K2		(1<<27)
#define JP_KNUM		(1<<28)
#define JP_K9		(1<<29)
#define JP_K6		(1<<30)
#define JP_K3		(1<<31)
#define JP_KR		JP_K6	/* pro controller buttons */
#define JP_KL		JP_K4
#define JP_KX		JP_K7
#define JP_KY		JP_K8
#define JP_KZ		JP_K9

/* --- Keyboard scancodes --- */
/* taken from svgalib/vgakeyboard.h */

#define SCANCODE_ESCAPE		0x01
#define SCANCODE_1		0x02
#define SCANCODE_2		0x03
#define SCANCODE_3		0x04
#define SCANCODE_4		0x05
#define SCANCODE_5		0x06
#define SCANCODE_6		0x07
#define SCANCODE_7		0x08
#define SCANCODE_8		0x09
#define SCANCODE_9		0x0a
#define SCANCODE_0		0x0b
#define SCANCODE_MINUS		0x0c
#define SCANCODE_EQUAL		0x0d
#define SCANCODE_BACKSPACE	0x0e

#define SCANCODE_TAB		0x0f
#define SCANCODE_Q		0x10
#define SCANCODE_W		0x11
#define SCANCODE_E		0x12
#define SCANCODE_R		0x13
#define SCANCODE_T		0x14
#define SCANCODE_Y		0x15
#define SCANCODE_U		0x16
#define SCANCODE_I		0x17
#define SCANCODE_O		0x18
#define SCANCODE_P		0x19
#define SCANCODE_BRACKET_LEFT	0x1a
#define SCANCODE_BRACKET_RIGHT	0x1b
#define SCANCODE_ENTER		0x1c
#define SCANCODE_DELETE		0x53

#define SCANCODE_LEFTCONTROL	0x1d
#define SCANCODE_A		0x1e
#define SCANCODE_S		0x1f
#define SCANCODE_D		0x20
#define SCANCODE_F		0x21
#define SCANCODE_G		0x22
#define SCANCODE_H		0x23
#define SCANCODE_J		0x24
#define SCANCODE_K		0x25
#define SCANCODE_L		0x26
#define SCANCODE_SEMICOLON	0x27
#define SCANCODE_APOSTROPHE	0x28
#define SCANCODE_GRAVE		0x29

#define SCANCODE_LEFTSHIFT	0x2a
#define SCANCODE_BACKSLASH	0x2b
#define SCANCODE_Z		0x2c
#define SCANCODE_X		0x2d
#define SCANCODE_C		0x2e
#define SCANCODE_V		0x2f
#define SCANCODE_B		0x30
#define SCANCODE_N		0x31
#define SCANCODE_M		0x32
#define SCANCODE_COMMA		0x33
#define SCANCODE_PERIOD		0x34
#define SCANCODE_SLASH		0x35
#define SCANCODE_RIGHTSHIFT	0x36

#define SCANCODE_LEFTALT	0x38
#define SCANCODE_SPACE		0x39
#define SCANCODE_CAPSLOCK	0x3a

#define SCANCODE_F1		0x3b
#define SCANCODE_F2		0x3c
#define SCANCODE_F3		0x3d
#define SCANCODE_F4		0x3e
#define SCANCODE_F5		0x3f
#define SCANCODE_F6		0x40
#define SCANCODE_F7		0x41
#define SCANCODE_F8		0x42
#define SCANCODE_F9		0x43
#define SCANCODE_F10		0x44

#define SCANCODE_PARENLEFT	0x63
#define SCANCODE_PARENRIGHT	0x64
#define SCANCODE_KEYPADDIVIDE	0x65
#define SCANCODE_KEYPADMULTIPLY	0x66
#define SCANCODE_KEYPAD7	0x67
#define SCANCODE_KEYPAD8	0x68
#define SCANCODE_KEYPAD9	0x69
#define SCANCODE_KEYPADMINUS	0x4a
#define SCANCODE_KEYPAD4	0x6a
#define SCANCODE_KEYPAD5	0x6b
#define SCANCODE_KEYPAD6	0x6c
#define SCANCODE_KEYPADPLUS	0x4e
#define SCANCODE_KEYPAD1	0x6d
#define SCANCODE_KEYPAD2	0x6e
#define SCANCODE_KEYPAD3	0x6f
#define SCANCODE_KEYPAD0	0x70
#define SCANCODE_KEYPADPERIOD	0x71
#define SCANCODE_KEYPADENTER	0x72

#define SCANCODE_HELP		0x62
#define SCANCODE_UNDO		0x61
/* disabled to emulate pc keys
#define SCANCODE_INSERT		0x52
#define SCANCODE_CLRHOME	0x47
*/
#define SCANCODE_CURSORBLOCKUP		0x48
#define SCANCODE_CURSORBLOCKLEFT	0x4b
#define SCANCODE_CURSORBLOCKDOWN	0x50
#define SCANCODE_CURSORBLOCKRIGHT	0x4d

/* Now codes to emulate missing PC keys */

#define SCANCODE_F11		SCANCODE_HELP
#define SCANCODE_F12		SCANCODE_UNDO
#define SCANCODE_BREAK		SCANCODE_PARENLEFT
#define SCANCODE_INSERT		SCANCODE_KEYPAD7
#define SCANCODE_HOME		SCANCODE_KEYPAD8
#define SCANCODE_PAGEUP		SCANCODE_KEYPAD9
#define SCANCODE_REMOVE		SCANCODE_KEYPAD4
#define SCANCODE_END		SCANCODE_KEYPAD5
#define SCANCODE_PAGEDOWN	SCANCODE_KEYPAD6

/* --- Time definitions --- */
/* uses 200hz system timer */

extern unsigned long atari_curtime,atari_basetime;

